@@ -257,10 +257,10 @@ _read_bytes (int      fd,
 
                 return FALSE;
         } else if (bytes_left_to_read > 0) {
-                g_set_error (error,
-                             G_FILE_ERROR,
-                             g_file_error_from_errno (errno),
-                             "%s", g_strerror (errno));
+                g_set_error_literal (error,
+                                     G_FILE_ERROR,
+                                     g_file_error_from_errno (errno),
+                                     g_strerror (errno));
                 return FALSE;
         }
 
@@ -292,10 +292,10 @@ gdm_generate_random_bytes (gsize    size,
         fd = open ("/dev/urandom", O_RDONLY);
 
         if (fd < 0) {
-                g_set_error (error,
-                             G_FILE_ERROR,
-                             g_file_error_from_errno (errno),
-                             "%s", g_strerror (errno));
+                g_set_error_literal (error,
+                                     G_FILE_ERROR,
+                                     g_file_error_from_errno (errno),
+                                     g_strerror (errno));
                 close (fd);
                 return NULL;
         }
@@ -321,8 +321,10 @@ gdm_generate_random_bytes (gsize    size,
         close (fd);
         return bytes;
 }
+
 static gboolean
 create_transient_display (GDBusConnection *connection,
+                          GCancellable    *cancellable,
                           GError         **error)
 {
         GError *local_error = NULL;
@@ -338,10 +340,10 @@ create_transient_display (GDBusConnection *connection,
                                              G_VARIANT_TYPE ("(o)"),
                                              G_DBUS_CALL_FLAGS_NONE,
                                              -1,
-                                             NULL, &local_error);
+                                             cancellable, &local_error);
         if (reply == NULL) {
                 g_warning ("Unable to create transient display: %s", local_error->message);
-                g_propagate_error (error, local_error);
+                g_propagate_prefixed_error (error, local_error, _("Unable to create transient display: "));
                 return FALSE;
         }
 
@@ -354,12 +356,17 @@ create_transient_display (GDBusConnection *connection,
 
 gboolean
 gdm_activate_session_by_id (GDBusConnection *connection,
+                            GCancellable    *cancellable,
                             const char      *seat_id,
                             const char      *session_id)
 {
         GError *local_error = NULL;
         GVariant *reply;
 
+        g_return_val_if_fail (G_IS_DBUS_CONNECTION (connection), FALSE);
+        g_return_val_if_fail (seat_id != NULL, FALSE);
+        g_return_val_if_fail (session_id != NULL, FALSE);
+
         reply = g_dbus_connection_call_sync (connection,
                                              "org.freedesktop.login1",
                                              "/org/freedesktop/login1",
@@ -369,7 +376,7 @@ gdm_activate_session_by_id (GDBusConnection *connection,
                                              NULL,
                                              G_DBUS_CALL_FLAGS_NONE,
                                              -1,
-                                             NULL, &local_error);
+                                             cancellable, &local_error);
         if (reply == NULL) {
                 g_warning ("Unable to activate session: %s", local_error->message);
                 g_error_free (local_error);
@@ -381,6 +388,38 @@ gdm_activate_session_by_id (GDBusConnection *connection,
         return TRUE;
 }
 
+gboolean
+gdm_terminate_session_by_id (GDBusConnection *connection,
+                             GCancellable    *cancellable,
+                             const char      *session_id)
+{
+        GError *local_error = NULL;
+        GVariant *reply;
+
+        g_return_val_if_fail (G_IS_DBUS_CONNECTION (connection), FALSE);
+        g_return_val_if_fail (session_id != NULL, FALSE);
+
+        reply = g_dbus_connection_call_sync (connection,
+                                             "org.freedesktop.login1",
+                                             "/org/freedesktop/login1",
+                                             "org.freedesktop.login1.Manager",
+                                             "TerminateSession",
+                                             g_variant_new ("(s)", session_id),
+                                             NULL,
+                                             G_DBUS_CALL_FLAGS_NONE,
+                                             -1,
+                                             cancellable, &local_error);
+        if (reply == NULL) {
+                g_warning ("Unable to terminate session: %s", local_error->message);
+                g_error_free (local_error);
+                return FALSE;
+        }
+
+        g_variant_unref (reply);
+
+        return TRUE;
+}
+
 gboolean
 gdm_get_login_window_session_id (const char  *seat_id,
                                  char       **session_id)
@@ -388,9 +427,8 @@ gdm_get_login_window_session_id (const char  *seat_id,
         gboolean   ret;
         int        res, i;
         char     **sessions;
-        char      *service_id;
-        char      *service_class;
-        char      *state;
+
+        g_return_val_if_fail (session_id != NULL, FALSE);
 
         res = sd_seat_get_sessions (seat_id, &sessions, NULL, NULL);
         if (res < 0) {
@@ -405,6 +443,9 @@ gdm_get_login_window_session_id (const char  *seat_id,
         }
 
         for (i = 0; sessions[i]; i ++) {
+                char *service_id = NULL;
+                char *service_class = NULL;
+                char *state = NULL;
 
                 res = sd_session_get_class (sessions[i], &service_class);
                 if (res < 0) {
@@ -423,8 +464,8 @@ gdm_get_login_window_session_id (const char  *seat_id,
 
                 free (service_class);
 
-                ret = sd_session_get_state (sessions[i], &state);
-                if (ret < 0) {
+                res = sd_session_get_state (sessions[i], &state);
+                if (res < 0) {
                         if (res == -ENXIO)
                                 continue;
 
@@ -477,6 +518,7 @@ out:
 
 static gboolean
 goto_login_session (GDBusConnection  *connection,
+                    GCancellable     *cancellable,
                     GError          **error)
 {
         gboolean        ret;
@@ -484,6 +526,7 @@ goto_login_session (GDBusConnection  *connection,
         char           *our_session;
         char           *session_id;
         char           *seat_id;
+        GError         *local_error = NULL;
 
         ret = FALSE;
         session_id = NULL;
@@ -496,10 +539,8 @@ goto_login_session (GDBusConnection  *connection,
          * since the data allocated is from libsystemd-logind, which
          * does not use GLib's g_malloc (). */
 
-        res = sd_pid_get_session (0, &our_session);
-        if (res < 0) {
-                g_debug ("failed to determine own session: %s", strerror (-res));
-                g_set_error (error, GDM_COMMON_ERROR, 0, _("Could not identify the current session."));
+        if (!gdm_find_display_session (0, getuid (), &our_session, &local_error)) {
+                g_propagate_prefixed_error (error, local_error, _("Could not identify the current session: "));
 
                 return FALSE;
         }
@@ -513,27 +554,9 @@ goto_login_session (GDBusConnection  *connection,
                 return FALSE;
         }
 
-        res = sd_seat_can_multi_session (seat_id);
-        if (res < 0) {
-                free (seat_id);
-
-                g_debug ("failed to determine whether seat can do multi session: %s", strerror (-res));
-                g_set_error (error, GDM_COMMON_ERROR, 0, _("The system is unable to determine whether to switch to an existing login screen or start up a new login screen."));
-
-                return FALSE;
-        }
-
-        if (res == 0) {
-                free (seat_id);
-
-                g_set_error (error, GDM_COMMON_ERROR, 0, _("The system is unable to start up a new login screen."));
-
-                return FALSE;
-        }
-
         res = gdm_get_login_window_session_id (seat_id, &session_id);
         if (res && session_id != NULL) {
-                res = gdm_activate_session_by_id (connection, seat_id, session_id);
+                res = gdm_activate_session_by_id (connection, cancellable, seat_id, session_id);
 
                 if (res) {
                         ret = TRUE;
@@ -541,7 +564,7 @@ goto_login_session (GDBusConnection  *connection,
         }
 
         if (! ret && g_strcmp0 (seat_id, "seat0") == 0) {
-                res = create_transient_display (connection, error);
+                res = create_transient_display (connection, cancellable, error);
                 if (res) {
                         ret = TRUE;
                 }
@@ -554,20 +577,20 @@ goto_login_session (GDBusConnection  *connection,
 }
 
 gboolean
-gdm_goto_login_session (GError **error)
+gdm_goto_login_session (GCancellable *cancellable,
+                        GError      **error)
 {
-        GError *local_error;
-        GDBusConnection *connection;
+        g_autoptr(GDBusConnection) connection = NULL;
+        g_autoptr(GError) local_error = NULL;
 
-        local_error = NULL;
-        connection = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, &local_error);
+        connection = g_bus_get_sync (G_BUS_TYPE_SYSTEM, cancellable, &local_error);
         if (connection == NULL) {
                 g_debug ("Failed to connect to the D-Bus daemon: %s", local_error->message);
-                g_propagate_error (error, local_error);
+                g_propagate_error (error, g_steal_pointer (&local_error));
                 return FALSE;
         }
 
-        return goto_login_session (connection, error);
+        return goto_login_session (connection, cancellable, error);
 }
 
 static void
@@ -591,7 +614,7 @@ gdm_get_script_environment (const char *username,
         GHashTable    *hash;
         struct passwd *pwent;
 
-        env = g_ptr_array_new ();
+        env = g_ptr_array_new_with_free_func (g_free);
 
         /* create a hash table of current environment, then update keys has necessary */
         hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
@@ -737,7 +760,6 @@ gdm_run_script (const char *dir,
                             &status,
                             &error);
 
-        g_ptr_array_foreach (env, (GFunc)g_free, NULL);
         g_ptr_array_free (env, TRUE);
         g_strfreev (argv);
 
@@ -843,3 +865,258 @@ gdm_shell_expand (const char *str,
         }
         return g_string_free (s, FALSE);
 }
+
+static gboolean
+_systemd_session_is_graphical (const char *session_id)
+{
+        const gchar * const graphical_session_types[] = { "wayland", "x11", "mir", NULL };
+        int saved_errno;
+        g_autofree gchar *type = NULL;
+
+        saved_errno = sd_session_get_type (session_id, &type);
+        if (saved_errno < 0) {
+                g_warning ("Couldn't get type for session '%s': %s",
+                           session_id,
+                           g_strerror (-saved_errno));
+                return FALSE;
+        }
+
+        if (!g_strv_contains (graphical_session_types, type)) {
+                g_debug ("Session '%s' is not a graphical session (type: '%s')",
+                         session_id,
+                         type);
+                return FALSE;
+        }
+
+        return TRUE;
+}
+
+static gboolean
+_systemd_session_is_active (const char *session_id)
+{
+        const gchar * const active_states[] = { "active", "online", NULL };
+        int saved_errno;
+        g_autofree gchar *state = NULL;
+
+        /*
+         * display sessions can be 'closing' if they are logged out but some
+         * processes are lingering; we shouldn't consider these (this is
+         * checking for a race condition since we specified
+         * GDM_SYSTEMD_SESSION_REQUIRE_ONLINE)
+         */
+        saved_errno = sd_session_get_state (session_id, &state);
+        if (saved_errno < 0) {
+                g_warning ("Couldn't get state for session '%s': %s",
+                           session_id,
+                           g_strerror (-saved_errno));
+                return FALSE;
+        }
+
+        if (!g_strv_contains (active_states, state)) {
+                g_debug ("Session '%s' is not active or online", session_id);
+                return FALSE;
+        }
+
+        return TRUE;
+}
+
+gboolean
+gdm_find_display_session (GPid        pid,
+                          const uid_t uid,
+                          char      **out_session_id,
+                          GError    **error)
+{
+        char *local_session_id = NULL;
+        g_auto(GStrv) sessions = NULL;
+        int n_sessions;
+        int res;
+
+        g_return_val_if_fail (out_session_id != NULL, FALSE);
+
+        /* First try to look up the session using the pid. We need this
+         * at least for the greeter, because it currently runs multiple
+         * sessions under the same user.
+         * See also commit 2b52d8933c8ab38e7ee83318da2363d00d8c5581
+         * which added an explicit dbus-run-session for all but seat0.
+         */
+        res = sd_pid_get_session (pid, &local_session_id);
+        if (res >= 0) {
+                g_debug ("GdmCommon: Found session %s for PID %d, using", local_session_id, pid);
+
+                *out_session_id = g_strdup (local_session_id);
+                free (local_session_id);
+
+                return TRUE;
+        } else {
+                if (res != -ENODATA)
+                        g_warning ("GdmCommon: Failed to retrieve session information for pid %d: %s",
+                                   pid, strerror (-res));
+        }
+
+        g_debug ("Finding a graphical session for user %d", uid);
+
+        n_sessions = sd_uid_get_sessions (uid,
+                                          GDM_SYSTEMD_SESSION_REQUIRE_ONLINE,
+                                          &sessions);
+
+        if (n_sessions < 0) {
+                g_set_error (error,
+                             GDM_COMMON_ERROR,
+                             0,
+                             "Failed to get sessions for user %d",
+                             uid);
+                return FALSE;
+        }
+
+        for (int i = 0; i < n_sessions; ++i) {
+                g_debug ("Considering session '%s'", sessions[i]);
+
+                if (!_systemd_session_is_graphical (sessions[i]))
+                        continue;
+
+                if (!_systemd_session_is_active (sessions[i]))
+                        continue;
+
+                /*
+                 * We get the sessions from newest to oldest, so take the last
+                 * one we find that's good
+                 */
+                local_session_id = sessions[i];
+        }
+
+        if (local_session_id == NULL) {
+                g_set_error (error,
+                             GDM_COMMON_ERROR,
+                             0,
+                             "Could not find a graphical session for user %d",
+                             uid);
+                return FALSE;
+        }
+
+        *out_session_id = g_strdup (local_session_id);
+
+        return TRUE;
+}
+
+static void
+load_env_file (GFile *file,
+               GdmLoadEnvVarFunc load_env_func,
+               GdmExpandVarFunc  expand_func,
+               gpointer user_data)
+{
+        gchar *contents;
+        gchar **lines;
+        gchar *line, *p;
+        gchar *var, *var_end;
+        gchar *expanded;
+        char *filename;
+        int i;
+
+        filename = g_file_get_path (file);
+        g_debug ("Loading env vars from %s\n", filename);
+        g_free (filename);
+
+        if (g_file_load_contents (file, NULL, &contents, NULL, NULL, NULL)) {
+                lines = g_strsplit (contents, "\n", -1);
+                g_free (contents);
+                for (i = 0; lines[i] != NULL; i++) {
+                        line = lines[i];
+                        p = line;
+                        while (g_ascii_isspace (*p))
+                                p++;
+                        if (*p == '#' || *p == '\0')
+                                continue;
+                        var = p;
+                        while (gdm_shell_var_is_valid_char (*p, p == var))
+                                p++;
+                        var_end = p;
+                        while (g_ascii_isspace (*p))
+                                p++;
+                        if (var == var_end || *p != '=') {
+                                g_warning ("Invalid env.d line '%s'\n", line);
+                                continue;
+                        }
+                        *var_end = 0;
+                        p++; /* Skip = */
+                        while (g_ascii_isspace (*p))
+                                p++;
+
+                        expanded = gdm_shell_expand (p, expand_func, user_data);
+                        expanded = g_strchomp (expanded);
+                        load_env_func (var, expanded, user_data);
+                        g_free (expanded);
+                }
+                g_strfreev (lines);
+        }
+}
+
+static gint
+compare_str (gconstpointer  a,
+             gconstpointer  b)
+{
+  return strcmp (*(const char **)a, *(const char **)b);
+}
+
+static void
+gdm_load_env_dir (GFile *dir,
+                  GdmLoadEnvVarFunc load_env_func,
+                  GdmExpandVarFunc  expand_func,
+                  gpointer user_data)
+{
+        GFileInfo *info = NULL;
+        GFileEnumerator *enumerator = NULL;
+        GPtrArray *names = NULL;
+        GFile *file;
+        const gchar *name;
+        int i;
+
+        enumerator = g_file_enumerate_children (dir,
+                                                G_FILE_ATTRIBUTE_STANDARD_TYPE","
+                                                G_FILE_ATTRIBUTE_STANDARD_NAME","
+                                                G_FILE_ATTRIBUTE_STANDARD_IS_HIDDEN","
+                                                G_FILE_ATTRIBUTE_STANDARD_IS_BACKUP,
+                                                G_FILE_QUERY_INFO_NONE,
+                                                NULL, NULL);
+        if (!enumerator) {
+                goto out;
+        }
+
+        names = g_ptr_array_new_with_free_func (g_free);
+        while ((info = g_file_enumerator_next_file (enumerator, NULL, NULL)) != NULL) {
+                if (g_file_info_get_file_type (info) == G_FILE_TYPE_REGULAR &&
+                    !g_file_info_get_is_hidden (info) &&
+                    g_str_has_suffix (g_file_info_get_name (info), ".env"))
+                  g_ptr_array_add (names, g_strdup (g_file_info_get_name (info)));
+
+                g_clear_object (&info);
+        }
+
+        g_ptr_array_sort (names, compare_str);
+
+        for (i = 0; i < names->len; i++) {
+                name = g_ptr_array_index (names, i);
+                file = g_file_get_child (dir, name);
+                load_env_file (file, load_env_func, expand_func, user_data);
+                g_object_unref (file);
+        }
+
+ out:
+        g_clear_pointer (&names, g_ptr_array_unref);
+        g_clear_object (&enumerator);
+}
+
+void
+gdm_load_env_d (GdmLoadEnvVarFunc load_env_func,
+                GdmExpandVarFunc  expand_func,
+                gpointer user_data)
+{
+        GFile *dir;
+
+        dir = g_file_new_for_path (DATADIR "/gdm/env.d");
+        gdm_load_env_dir (dir, load_env_func, expand_func, user_data);
+        g_object_unref (dir);
+
+        dir = g_file_new_for_path (GDMCONFDIR "/env.d");
+        gdm_load_env_dir (dir, load_env_func, expand_func, user_data);
+        g_object_unref (dir);
+}
