@@ -18,6 +18,8 @@
  *
  */
 
+#include "config.h"
+
 #include "gdm-dbus-util.h"
 #include <string.h>
 
@@ -52,8 +54,8 @@ handle_connection (GDBusServer      *server,
                    GDBusConnection  *new_connection,
                    gpointer          user_data)
 {
+        g_autoptr(GDBusNodeInfo) bus_info = NULL;
         GDBusInterfaceVTable bus_vtable = { handle_bus_method };
-        GDBusNodeInfo *bus_info;
 
         bus_info = g_dbus_node_info_new_for_xml (dbus_introspection,
                                                  NULL);
@@ -65,7 +67,6 @@ handle_connection (GDBusServer      *server,
                                            bus_info->interfaces[0],
                                            &bus_vtable,
                                            NULL, NULL, NULL);
-        g_dbus_node_info_unref (bus_info);
 
         /* We're not handling the signal */
         return FALSE;
@@ -75,13 +76,23 @@ GDBusServer *
 gdm_dbus_setup_private_server (GDBusAuthObserver  *observer,
                                GError            **error)
 {
-        char *guid;
+        g_autofree char *guid = NULL;
         const char *client_address;
-        GDBusServer *server;
+        g_autoptr(GDBusServer) server = NULL;
+
+        /* Ensure the private bus directory exists */
+        if (g_mkdir_with_parents (GDM_PRIVATE_DBUS_DIR, 0711) == -1) {
+                g_set_error_literal (error, G_IO_ERROR, g_io_error_from_errno (errno), g_strerror (errno));
+                return NULL;
+        }
+        if (g_chmod (GDM_PRIVATE_DBUS_DIR, 0711) == -1) {
+                g_set_error_literal (error, G_IO_ERROR, g_io_error_from_errno (errno), g_strerror (errno));
+                return NULL;
+        }
 
         guid = g_dbus_generate_guid ();
 
-        server = g_dbus_server_new_sync ("unix:tmpdir=/tmp",
+        server = g_dbus_server_new_sync ("unix:dir=" GDM_PRIVATE_DBUS_DIR,
                                          G_DBUS_SERVER_FLAGS_NONE,
                                          guid,
                                          observer,
@@ -99,9 +110,7 @@ gdm_dbus_setup_private_server (GDBusAuthObserver  *observer,
                           G_CALLBACK (handle_connection),
                           NULL);
 
-        g_free (guid);
-
-        return server;
+        return g_steal_pointer (&server);
 }
 
 gboolean
@@ -109,11 +118,13 @@ gdm_dbus_get_pid_for_name (const char  *system_bus_name,
                            pid_t       *out_pid,
                            GError     **error)
 {
-        GDBusConnection *bus;
-        GVariant *reply;
-        gboolean retval = FALSE;
+        g_autoptr(GDBusConnection) bus = NULL;
+        g_autoptr(GVariant) reply = NULL;
         unsigned int v;
 
+        g_return_val_if_fail (system_bus_name != NULL, FALSE);
+        g_return_val_if_fail (out_pid != NULL, FALSE);
+
         bus = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, error);
         if (bus == NULL) {
                 return FALSE;
@@ -130,18 +141,13 @@ gdm_dbus_get_pid_for_name (const char  *system_bus_name,
                                              -1,
                                              NULL, error);
         if (reply == NULL) {
-                goto out;
+                return FALSE;
         }
 
         g_variant_get (reply, "(u)", &v);
         *out_pid = v;
-        g_variant_unref (reply);
 
-        retval = TRUE;
- out:
-        g_object_unref (bus);
-
-        return retval;
+        return TRUE;
 }
 
 gboolean
@@ -149,11 +155,13 @@ gdm_dbus_get_uid_for_name (const char  *system_bus_name,
                            uid_t       *out_uid,
                            GError     **error)
 {
-        GDBusConnection *bus;
-        GVariant *reply;
-        gboolean retval = FALSE;
+        g_autoptr(GDBusConnection) bus = NULL;
+        g_autoptr(GVariant) reply = NULL;
         unsigned int v;
 
+        g_return_val_if_fail (system_bus_name != NULL, FALSE);
+        g_return_val_if_fail (out_uid != NULL, FALSE);
+
         bus = g_bus_get_sync (G_BUS_TYPE_SYSTEM, NULL, error);
         if (bus == NULL) {
                 return FALSE;
@@ -170,18 +178,13 @@ gdm_dbus_get_uid_for_name (const char  *system_bus_name,
                                              -1,
                                              NULL, error);
         if (reply == NULL) {
-                goto out;
+                return FALSE;
         }
 
         g_variant_get (reply, "(u)", &v);
         *out_uid = v;
-        g_variant_unref (reply);
 
-        retval = TRUE;
- out:
-        g_object_unref (bus);
-
-        return retval;
+        return TRUE;
 }
 
 void
