@@ -179,14 +179,14 @@ gdm_daemon_ensure_dirs (uid_t uid,
 
         /* Set up /var/run/gdm */
         if (!ensure_dir_with_perms (GDM_RAN_ONCE_MARKER_DIR, 0, gid, 0711, &error)) {
-                gdm_fail (_("Failed to create ran once marker dir %s: %s"),
-                          GDM_RAN_ONCE_MARKER_DIR, error->message);
+                g_critical (_("Failed to create ran once marker dir %s: %s"),
+                            GDM_RAN_ONCE_MARKER_DIR, error->message);
         }
 
         /* Set up /var/log/gdm */
         if (!ensure_dir_with_perms (LOGDIR, 0, gid, 0711, &error)) {
-                gdm_fail (_("Failed to create LogDir %s: %s"),
-                          LOGDIR, error->message);
+                g_critical (_("Failed to create LogDir %s: %s"),
+                            LOGDIR, error->message);
         }
 }
 
@@ -220,25 +220,25 @@ gdm_daemon_lookup_user (uid_t *uidp,
 
         /* Set uid and gid */
         if G_UNLIKELY (pwent == NULL) {
-                gdm_fail (_("Can’t find the GDM user “%s”. Aborting!"), username);
+                g_critical (_("Can’t find the GDM user “%s”. Aborting!"), username);
         } else {
                 uid = pwent->pw_uid;
         }
 
         if G_UNLIKELY (uid == 0) {
-                gdm_fail (_("The GDM user should not be root. Aborting!"));
+                g_critical (_("The GDM user should not be root. Aborting!"));
         }
 
         grent = getgrnam (groupname);
 
         if G_UNLIKELY (grent == NULL) {
-                gdm_fail (_("Can’t find the GDM group “%s”. Aborting!"), groupname);
+                g_critical (_("Can’t find the GDM group “%s”. Aborting!"), groupname);
         } else  {
                 gid = grent->gr_gid;
         }
 
         if G_UNLIKELY (gid == 0) {
-                gdm_fail (_("The GDM group should not be root. Aborting!"));
+                g_critical (_("The GDM group should not be root. Aborting!"));
         }
 
         if (uidp != NULL) {
@@ -267,16 +267,8 @@ static gboolean
 on_sighup_cb (gpointer user_data)
 {
         g_debug ("Got HUP signal");
-        /* Reread config stuff like system config files, VPN service
-         * files, etc
-         */
-        g_object_unref (settings);
-        settings = gdm_settings_new ();
-        if (settings != NULL) {
-                if (! gdm_settings_direct_init (settings, DATADIR "/gdm/gdm.schemas", "/")) {
-                        g_warning ("Unable to initialize settings");
-                }
-        }
+
+        gdm_settings_reload (settings);
 
         return TRUE;
 }
@@ -329,7 +321,6 @@ main (int    argc,
 
         context = g_option_context_new (_("GNOME Display Manager"));
         g_option_context_add_main_entries (context, entries, NULL);
-        g_option_context_set_ignore_unknown_options (context, TRUE);
 
         error = NULL;
         res = g_option_context_parse (context, &argc, &argv, &error);
@@ -415,6 +406,7 @@ on_name_acquired (GDBusConnection *bus,
 {
         gboolean xdmcp_enabled;
         gboolean show_local_greeter;
+        gboolean remote_login_enabled;
 
         manager = gdm_manager_new ();
         if (manager == NULL) {
@@ -432,6 +424,10 @@ on_name_acquired (GDBusConnection *bus,
         gdm_settings_direct_get_boolean (GDM_KEY_XDMCP_ENABLE, &xdmcp_enabled);
         gdm_manager_set_xdmcp_enabled (manager, xdmcp_enabled);
 
+        remote_login_enabled = FALSE;
+        gdm_settings_direct_get_boolean (GDM_KEY_REMOTE_LOGIN_ENABLE, &remote_login_enabled);
+        gdm_manager_set_remote_login_enabled (manager, remote_login_enabled);
+
         gdm_manager_start (manager);
 }
 
